*********************************************************************
      PROGRAM sumtally
*                                                                   *
*         Created by S. Hashimoto on 2014/03/12                     *
*                                                                   *
*-------------------------------------------------------------------*
*                                                                   *
*     Purpose                                                       *
*         Obtain sum of results of tally                            *
*     Required files for run                                        *
*         Input file for this program                               *
*         Results of tally                                          *
*     Outputs                                                       *
*         Tally file                                                *
*                                                                   *
*********************************************************************
      implicit none

      integer :: stdin=5,stdout=6,io=10
      integer, parameter :: nfmax=40, nrdatamax=100000
      integer :: ifile, nfile, i1, i2, i3, ios, nlength
      character(100) :: afileout, afile(nfmax)
      real(8) :: weight(nfmax), totweight
      logical Fex

      integer :: iread
      integer, parameter :: nread=10000
      character(200) :: aread, cnum
      character(4) :: caxis='axis'
      character(2) :: cx=' x', cy=' y', cz=' z', cr=' r', ct=' t'
      character(2) :: cxy='xy', cyz='yz', cxz='xz', crz='rz'
      integer :: iaxis
      character(2) :: cnx='nx', cny='ny', cnz='nz'
      integer :: nx, ny, nz, nx0, ny0, nz0, nmesh
      character(6) :: ckey='hc:  y', ckey2='h: n  '
      integer :: nrdata, icol
      real(8) :: rdata(10,nrdatamax,nfmax), sdata(10,nrdatamax)
      real(8) :: erdata(10,nrdatamax,nfmax), esdata(10,nrdatamax)
      character(6) :: cistdev='istdev', cmaxcas='maxcas'
      character(5) :: cresc2='resc2', cresc3='resc3'
      real(8) :: resc3(nfmax), tothis

      character(4) :: cxorg='xorg'

!----------------------------------------------------------------------
c$$$      open(stdin,file='sumtally.inp')
      read(stdin,'(a)') afileout
      i1=index(afileout,'"')
      i2=index(afileout,'"',back=.true.)
      open(stdout,file=afileout(i1+1:i2-1))
      read(stdin,*) nfile
      if(nfile.gt.nfmax) then
         write(*,*)'ERROR: number of tally files is larger than',nfmax
         stop
      end if
      do ifile=1,nfile
         read(stdin,'(e10.0,a)',iostat=ios) weight(ifile),afile(ifile)
         if (ifile.ne.nfile .and. ios.lt.0) then
            write(*,*)'ERROR: number of tally files does not equal to'
     1           ,nfile
            stop
         end if
      end do

      totweight=0d0
      do ifile=1,nfile
         totweight=totweight+weight(ifile)
      end do
      do ifile=1,nfile
         weight(ifile)=weight(ifile)/totweight
      end do

      iaxis=0

* start of reading files of tally
      do ifile=1,nfile

      i1=index(afile(ifile),'"')
      i2=index(afile(ifile),'"',back=.true.)
      inquire(file=afile(ifile)(i1+1:i2-1),exist=Fex)
      if(.not.Fex)then
        write(*,*)'ERROR: missing files of result of tally'
        stop
      endif
      open(io,file=afile(ifile)(i1+1:i2-1))

      do iread=1,nread
         read(io,'(a)') aread
         if(aread(16:17).eq.cxy .or. aread(16:17).eq.cyz
     1        .or. aread(16:17).eq.cxz .or. aread(16:17).eq.crz) then
            iaxis=2
         else if(aread(16:17).eq.cx .or. aread(16:17).eq.cy
     1           .or. aread(16:17).eq.cz .or. aread(16:17).eq.cr
     2           .or. aread(16:17).eq.ct) then
            iaxis=1
         end if
         if(aread(8:9).eq.cnx) then
            i1=index(aread,'=')
            i2=index(aread,'#')
            write(cnum,'(a)') aread(i1+1:i2-1)
            read(cnum,'(i17)') nx
         end if
         if(aread(8:9).eq.cny) then
            i1=index(aread,'=')
            i2=index(aread,'#')
            write(cnum,'(a)') aread(i1+1:i2-1)
            read(cnum,'(i17)') ny
         end if
         if(aread(8:9).eq.cnz) then
            i1=index(aread,'=')
            i2=index(aread,'#')
            write(cnum,'(a)') aread(i1+1:i2-1)
            read(cnum,'(i17)') nz
         end if
         nlength=len_trim(aread)
         if(ifile.eq.1) write(stdout,'(a)') aread(1:nlength)
         if(aread(1:6).eq.ckey .or. aread(1:6).eq.ckey2) goto 1000
      end do
 1000 continue

      if(ifile.eq.1) then
         nx0=nx
         ny0=ny
         nz0=nz
      else
         if(nx.ne.nx0 .or. ny.ne.ny0 .or. nz.ne.nz0) then
            write(stdout,*)'ERROR: no match numbers of xyz mesh'
            stop
         end if
      end if

* for axis=xy,yz,xz,rz
      if(iaxis.eq.2) then
         nmesh=nx*ny*nz
         nrdata=nmesh/10
         if(nrdata.gt.nrdatamax) then
            write(*,*)'ERROR: nx*xy*nz is larger than',nrdatamax
            stop
         end if
      do iread=1,nrdata
         read(io,'(1p10e11.3)') (rdata(icol,iread,ifile),icol=1,10)
      end do

* for axis=x,y,z,r,t
      else if(iaxis.eq.1) then
         read(io,'(a)') aread
         nlength=len_trim(aread)
         if(ifile.eq.1) write(stdout,'(a)') aread(1:nlength)

         nrdata=max0(nx,ny,nz)
         if(nrdata.gt.nrdatamax) then
            write(*,*)'ERROR: nx, xy, nz are larger than',nrdatamax
            stop
         end if
         do iread=1,nrdata
            read(io,'(1p3e13.4,1f8.4)')
     1           (rdata(icol,iread,ifile),icol=1,3)
     2           ,erdata(1,iread,ifile)
         end do

      end if

      do iread=1,nread
         read(io,'(a)',end=2000) aread
         if(aread(3:7).eq.cresc3) then
            i1=index(aread,'=')
            i2=index(aread,'# T')
            write(cnum,'(a)') aread(i1+1:i2-1)
            read(cnum,'(e25.17)') resc3(ifile)
         end if
      end do
 2000 continue

      end do
* end of reading files of tally


      if(iaxis.eq.1) goto 5000 ! for axis=x,y,z,r,t
!----------------------------------------------------------------------
* summation of results of tally for axis=xy,yz,xz,rz
      do iread=1,nrdata
      do icol=1,10
         sdata(icol,iread)=0d0
      end do
      end do

c$$$         tothis=0d0
      do ifile=1,nfile
      do iread=1,nrdata
      do icol=1,10
         sdata(icol,iread)=sdata(icol,iread)
c$$$     1        +rdata(icol,iread,ifile)*resc3(ifile)*weight(ifile)
     1        +rdata(icol,iread,ifile)*weight(ifile)
      end do
      end do
c$$$         tothis=tothis+resc3(ifile)
      end do

c$$$      do iread=1,nrdata
c$$$      do icol=1,10
c$$$         sdata(icol,iread)=sdata(icol,iread)/tothis
c$$$      end do
c$$$      end do


* write sum of results of tally
      do iread=1,nrdata
         write(stdout,'(1p10e11.3)') (sdata(icol,iread),icol=1,10)
      end do


* write last of tally file
      do ifile=1,nfile
      i1=index(afile(ifile),'"')
      i2=index(afile(ifile),'"',back=.true.)
      open(io,file=afile(ifile)(i1+1:i2-1))
      write(stdout,'(//a)') '#------------------------------'
      write(stdout,'(a,a)') '# footer of ', afile(ifile)(i1+1:i2-1)
      write(stdout,'(a)') '#------------------------------'

      do iread=1,nread
         read(io,'(a)') aread
         if(aread(1:6).eq.ckey) goto 3000
      end do
 3000 continue
      do iread=1,nrdata
         read(io,*)
      end do
      do iread=1,nread
         read(io,'(a)',end=4000) aread
         if(ifile.ne.1 .and. aread(4:7).eq.cxorg) goto 4000
         nlength=len_trim(aread)
         write(stdout,'(a)') aread(1:nlength)
      end do
 4000 continue

      end do
      close(stdout)

!----------------------------------------------------------------------
* error file for axis=xy,yz,xz,rz
      if (iaxis.eq.2) then
      i1=index(afileout,'"')
      i2=index(afileout,'"',back=.true.)
      i3=index(afileout,'.',back=.true.)
      open(stdout,file=afileout(i1+1:i3-1)//'_err'//afileout(i3:i2-1))


* start of reading error files of tally
      do ifile=1,nfile

      i1=index(afile(ifile),'"')
      i2=index(afile(ifile),'"',back=.true.)
      i3=index(afile(ifile),'.',back=.true.)
      inquire(file
     1     =afile(ifile)(i1+1:i3-1)//'_err'//afile(ifile)(i3:i2-1)
     2     ,exist=Fex)
      if(.not.Fex)then
        write(*,*)'ERROR: missing error files of result of tally'
        stop
      endif
      open(io,file
     1     =afile(ifile)(i1+1:i3-1)//'_err'//afile(ifile)(i3:i2-1))

      do iread=1,nread
         read(io,'(a)') aread
         nlength=len_trim(aread)
         if(ifile.eq.1) write(stdout,'(a)') aread(1:nlength)
         if(aread(1:6).eq.ckey) goto 1001
      end do
 1001 continue

      do iread=1,nrdata
         read(io,'(1p10e11.3)') (erdata(icol,iread,ifile),icol=1,10)
      end do

      end do
* end of reading error files of tally


* summation of error results of tally
      do iread=1,nrdata
      do icol=1,10
         esdata(icol,iread)=0d0
      end do
      end do

      do ifile=1,nfile
      do iread=1,nrdata
      do icol=1,10
         esdata(icol,iread)=esdata(icol,iread)
     1        +(rdata(icol,iread,ifile)*erdata(icol,iread,ifile))**2
     2        *weight(ifile)**2
      end do
      end do
      end do

      do iread=1,nrdata
      do icol=1,10
      if(sdata(icol,iread).eq.0d0) then
         esdata(icol,iread)=dsqrt(esdata(icol,iread))
      else
         esdata(icol,iread)=dsqrt(esdata(icol,iread))/sdata(icol,iread)
      end if
      end do
      end do


* write sum of error results of tally
      do iread=1,nrdata
         write(stdout,'(1p10e11.3)') (esdata(icol,iread),icol=1,10)
      end do


* write last of error file
      do ifile=1,nfile
      i1=index(afile(ifile),'"')
      i2=index(afile(ifile),'"',back=.true.)
      i3=index(afile(ifile),'.',back=.true.)
      open(io,file
     1     =afile(ifile)(i1+1:i3-1)//'_err'//afile(ifile)(i3:i2-1))
      write(stdout,'(//a)') '#------------------------------'
      write(stdout,'(a,a)') '# footer of ', afile(ifile)(i1+1:i2-1)
      write(stdout,'(a)') '#------------------------------'

      do iread=1,nread
         read(io,'(a)') aread
         if(aread(1:6).eq.ckey) goto 3001
      end do
 3001 continue
      do iread=1,nrdata
         read(io,*)
      end do
      do iread=1,nread
         read(io,'(a)',end=4001) aread
         if(ifile.ne.1 .and. aread(4:7).eq.cxorg) goto 4001
         nlength=len_trim(aread)
         write(stdout,'(a)') aread(1:nlength)
      end do
 4001 continue

      end do
      close(stdout)


      end if


 5000 continue
!----------------------------------------------------------------------
* for axis=x,y,z,r,t
      if (iaxis.eq.1) then

* summation of results of tally
      do iread=1,nrdata
         sdata(3,iread)=0d0
         esdata(1,iread)=0d0
      end do

      do ifile=1,nfile
      do iread=1,nrdata
         sdata(3,iread)=sdata(3,iread)
     1        +rdata(3,iread,ifile)*weight(ifile)
      end do
      end do

      do ifile=1,nfile
      do iread=1,nrdata
         esdata(1,iread)=esdata(1,iread)
     1        +(rdata(3,iread,ifile)*erdata(1,iread,ifile))**2
     2        *weight(ifile)**2
      end do
      end do
      do iread=1,nrdata
      if(sdata(3,iread).eq.0d0) then
         esdata(1,iread)=dsqrt(esdata(1,iread))
      else
         esdata(1,iread)=dsqrt(esdata(1,iread))/sdata(3,iread)
      end if
      end do


* write sum of results of tally
      do iread=1,nrdata
         write(stdout,'(1p3e13.4,1f8.4)')
     1           (rdata(icol,iread,1),icol=1,2)
     2           ,sdata(3,iread),esdata(1,iread)
      end do


* write last of tally file
      do ifile=1,1!nfile
      i1=index(afile(ifile),'"')
      i2=index(afile(ifile),'"',back=.true.)
      open(io,file=afile(ifile)(i1+1:i2-1))
      write(stdout,'(//a)') '#------------------------------'
      write(stdout,'(a,a)') '# footer of ', afile(ifile)(i1+1:i2-1)
      write(stdout,'(a)') '#------------------------------'

      do iread=1,nread
         read(io,'(a)') aread
         if(aread(1:6).eq.ckey2) goto 5001
      end do
 5001 continue
         read(io,'(a)') aread
      do iread=1,nrdata
         read(io,*)
      end do
      do iread=1,nread
         read(io,'(a)',end=5002) aread
         nlength=len_trim(aread)
         write(stdout,'(a)') aread(1:nlength)
      end do
 5002 continue

      end do
      close(stdout)


      end if


!----------------------------------------------------------------------
      end PROGRAM sumtally
!----------------------------------------------------------------------
